function printrule(fls,method,filename)
// ----------------------------------------------------------------------
// Show in screen or write to file fls rules
// ----------------------------------------------------------------------
// This file is part of sciFLT ( Scilab Fuzzy Logic Toolbox )
// Copyright (C) @YEARS@ Jaime Urzua Grez
// mailto:jaime_urzua@yahoo.com
// 
// 2011 Holger Nahrstaedt
// ----------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// ----------------------------------------------------------------------
// CHANGES:
// 2004-11-08 Spell check.
// ----------------------------------------------------------------------

// CHECK INPUTS
if (argn(2)>0) then
	if (typeof(fls)~="fls") then
		error("printrule need a fls structure!");
	end
else
	error("printule need at least 1 parameter.");
end

// SET DEFAULT METHOD AND CHECK
if (argn(2)<2) then
	method="ling";
end

if ((method~="ling")&(method~="symb")&(method~="indx")) then
	error("Unknow method.");
end


// CHECK RULES
[ok,mes]=checkrule(fls);

if (ok~=0) then
	// THE RULES HAVE PROBLEMS
	write(%io(2),'The rules have problems... '+mes);
	
else
	// THE RULES ARE OK
	rr=1;
	if (argn(2)==3) then
		toscreen=%f;
		[u,rr]=mopen(filename,'w');
		if (rr~=0) then
			error('Something is wrong with file output.');
		end
		write(%io(2),'Please wait.');
	else
		toscreen=%t;
	end
	
	rule=fls.rule;
	ninputs=length(fls.input);
	noutputs=length(fls.output);
	nrules=size(rule,1);
	

	for j=1:nrules,
		// IF PART	
		if ((method=="ling")|(method=="symb")) then
			str="R"+string(j)+":";
			if (method=="ling") then
				str=str+" IF";
			end
		else
			str=string(j)+":";
		end
		
		me=rule(j,$-1);
		we=rule(j,$);
		docon=%f;
		for k=1:ninputs,
			idx=rule(j,k);
			if (idx<0) then
				docomplement=%t;
			else
				docomplement=%f;
			end
			idx=abs(idx);
			if (idx>0) then
				if (docon) then
					if (k~=1) then
						if (method=="ling") then
							if (me==0) then
								str=str+" OR";
							else
								str=str+" AND";
							end							
						else 
							if (me==0) then
								str=str+" |";
							else
								str=str+" &";
							end						
						end
					end
				end
				docon=%t;
				if (method=="ling") then
					str=str+" ("+string(fls.input(k).name);
					if (docomplement) then
						str=str+" ISN''T ";
					else
						str=str+" IS ";
					end
					str=str+string(fls.input(k).mf(idx).name)+")";
				elseif (method=="symb")
					str=str+" ("+string(fls.input(k).name);
					if (docomplement) then
						str=str+"~=";
					else
						str=str+"==";
					end
					str=str+string(fls.input(k).mf(idx).name)+")";
				else
					str=str+" (I_"+string(k);
					if (docomplement) then
						str=str+"~=";
					else
						str=str+"==";
					end
					str=str+"mf_"+string(idx)+")";
				end
			end
		end

		// THEN PART
		if (method=="ling") then
			str=str+" THEN ";
		else
			str=str+" => ";
		end		
		docon=%f;
		for k=1:noutputs,
			idx=rule(j,k+ninputs);
			if (idx<0) then
				docomplement=%t;
			else
				docomplement=%f;
			end
			idx=abs(idx);
			if ((docon)&(k~=noutputs)) then
				if ((method=="ling")|(method=="symb")) then
					str=str+" , ";
				else
					str=str+" ";				
				end
			end

			if (idx>0) then
				if (method=="ling") then
					str=str+"("+string(fls.output(k).name)+" IS "+string(fls.output(k).mf(idx).name)+")";
				elseif (method=="symb") then
					str=str+"("+string(fls.output(k).name)+"="+string(fls.output(k).mf(idx).name)+")";
				else
					str=str+"(O_"+string(k)+"=mf_"+string(idx)+")";
				end
			end
		end

		// WEIGTH PART
		if (method=="ling") then
			str=str+" weigth="+string(rule(k,$));
		else
			str=str+" ["+string(rule(k,$))+"]";
		end

		// PUT THE RESULT IN THE SCREEN OR IN A FILE
		if (toscreen) then
			write(%io(2),str);
		else 
			mputstr(str,u);
			mfprintf(u,"\n");			
		end
	end

	// CLOSE THE FILE IF THE OUTPUT IS A FILE
	if (~toscreen) then
		mclose(u);
	end
end

endfunction

